/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef __NODE_LIST_SIMPLE_H__
#define __NODE_LIST_SIMPLE_H__ 1

#include "node_list.h"
#include <pthread.h>

struct nls_node {
  wfs_node node;
  nls_node *prev;
  nls_node *next;
};

class node_list_simple : public node_list {

 private:
  nls_node *_nodes;
  pthread_mutex_t _nodes_mutex;

  nls_node *successor_node( wfs_node_id node_id, int n = 1 );
  void add_node_locked( wfs_node node );

 public:

  node_list_simple( wfs_node me );
  ~node_list_simple();
  wfs_node successor( wfs_node_id node_id, int n = 1 );
  wfs_node predecessor( wfs_node_id node_id );
  void add_node( wfs_node node );
  void sync_to_node_list( std::vector<wfs_node> nodes );
  wfs_node get_node( int index );

};

#endif // __NODE_LIST_SIMPLE_H__
