/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#include "replication_policy.h"
#include "marshall.h"
#include <string>
#include <iostream>

const std::string slice_assignment::to_canonical_string(void) const
{
  std::ostringstream ost;
  std::string s;
  // All the work done by the stream serizlization.
  ost << *this;
  s = ost.str();
  // strip newline
  size_t found = s.find_last_not_of('\n');
  s.erase(found+1);
  return s;
}
void slice_assignment::from_canonical_string(const std::string state)
{
  if(state != ""){
    std::istringstream ist(state);
    ist >> *this;
  }
}

bool slice_assignment::validate(std::string& err) const
{
  //TODO: implement
  return false;
}

slice_assignment::slice_assignment(std::string state, unsigned int exp)
:  lease_period(exp)
{
  from_canonical_string(state);
}


std::ostream& operator<<(std::ostream& ost, const rep_policy& rp)
{
  ost << rp.rep_num << " ";
  ost << static_cast<int>(rp.site_pol)  << " ";
  ost << rp.prim_site  << " ";
  ost << rp.site_num << " ";
  return ost;
}
std::istream& operator>> (std::istream& ist, rep_policy& rp)
{
  int i;
  ist >> rp.rep_num;
  ist >> i;
  rp.site_pol = static_cast<rep_policy::site_placement>(i);
  ist >> rp.prim_site;
  ist >> rp.site_num;
  return ist;
}

std::ostream& operator<<(std::ostream& ost, const slice_assignment& sa)
{
  // slice name
  ost << std::hex << sa.slice << std::dec << " ";
  // slice policy
  ost << sa.policy;
  // Replica list size
  ost << sa.replicas.size() << " ";
  // replica list
  for (std::list<host>::const_iterator it = sa.replicas.begin(); it != sa.replicas.end(); it++){
    // Write the host field by field to get the site_id
    ost << std::hex << it->addr << std::dec << " " << it->port << " " <<  it->site_id << " ";
  }
  // Terminate with a newline
  ost << std::endl;

  return ost;
}
std::istream& operator>> (std::istream& ist, slice_assignment& sa)
{
  unsigned int addr,port;
  std::string site;
  int size;
  ist >> std::hex >> sa.slice >> std::dec;
  ist >> sa.policy;
  ist >> size;
  sa.replicas.clear();
  for (int i=0;i< size;++i){
    ist >> std::hex >> addr >> std::dec; ist >> port; ist >> site;
    host h(addr,port,site);
    sa.replicas.push_back(h);
  }
  return ist;
}

marshall &
operator<<(marshall &m, slice_assignment sa)
{
  std::ostringstream ost;
  std::string s;
  // All the work done by the stream serialization.
  ost << sa;
  s = ost.str();
  m << s;
  return m;
}
unmarshall &
operator>>(unmarshall &u, slice_assignment& sa)
{
  std::string s;
  u >> s;
  std::istringstream ist(s);
  ist >> sa;
  return u;
}

rep_policy::operator std::string()
{
  std::ostringstream ost;
  ost << *this;
  return ost.str();
}


