/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


/*
 * slice_lock.h
 *
 */
#ifndef SLICE_LOCK_H_
#define SLICE_LOCK_H_

#include "wfs_slice.h"
#include "rsm.h"

// TODO: Move to its own header and source file
class slice_lock {

public:

  typedef unsigned int lease_t;

  enum lavail {
    FREE,
    OWNED
  };

  slice_lock(wfs_slice_id n = 0) : name(n), avail(FREE), state(""),
    lease_period(0), granted_on(0) { };
  slice_lock(const slice_lock & sl);
  wfs_slice_id name;
  lavail avail;
  std::string state;  // Opaque string representing the policy
  unsigned lease_period; // Lock lease time in seconds
  /*  granted_on:
   *  When was granted. Seconds from the Epoch as recorded by the master.
   *  This is only updated on the master and zero an all slaves.
   */
  lease_t granted_on;
  host primary;
  /*
   * We use the rsm viewstamps to update when this lock has changed.
   * This is only maintained on the master and used for efficient syncing
   * with the clients. This field is only updated on replicated operations
   * and is not marshaled in the state transfer.
   */
  viewstamp last_modified;
  int ttexpire() const {return (granted_on)?(lease_period - (time(NULL) - granted_on)):(-1);};
  const std::string toString() const;  // TODO: add string caching?
};

std::ostream & operator<<(std::ostream &os,const slice_lock &l);
std::istream & operator>>(std::istream &is, slice_lock &l);

marshall & operator<<(marshall &m, slice_lock l);
unmarshall & operator>>(unmarshall &u, slice_lock &l);

#endif /* SLICE_LOCK_H_ */
