/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef __WFS_SLICE_H
#define __WFS_SLICE_H 1

#include <iostream>
#include "wfs_file.h"
#include "rpc.h"
#include "marshall.h"

typedef wfs_file_id wfs_slice_id;

// I wrap the value with a class to ensure it has only the 20 MSB lit.
class wfs_slice {
  private:
    wfs_slice_id _val;

  public:
    static const unsigned int SLICE_BIT_COUNT = 20;
    static const wfs_slice_id SLICE_MASK = ~((wfs_slice_id)0) << ((sizeof(wfs_slice_id)*8) - SLICE_BIT_COUNT);
    static const unsigned int MAX_SLICE = SLICE_MASK >> ((sizeof(wfs_slice_id)*8) - SLICE_BIT_COUNT);
    static const wfs_slice_id ROOT_SLICE = ((wfs_slice_id)0);
    explicit wfs_slice(wfs_file_id fid = 0) {_val = fid & SLICE_MASK;} // This works for the current u32int wfs_file_id implementation
    wfs_slice(const wfs_slice& sl) : _val(sl._val) {}
    bool contains(const wfs_file_id fid){return (_val == (fid & SLICE_MASK));}
    wfs_slice_id& val()  {return _val;}
    wfs_slice_id val() const {return _val;}

    // Conversion to wfs_slice_id (unsigned int)
    operator wfs_slice_id() const { return _val; }
    static wfs_slice_id get_slice_id(wfs_file_id fid) {return fid & SLICE_MASK;}
};

bool operator==(wfs_slice a,wfs_slice b);
bool operator!=(wfs_slice a, wfs_slice b);
std::istream& operator>>(std::istream& is, wfs_slice& slice);
std::ostream& operator<<(std::ostream& os, const wfs_slice slice);

inline unmarshall &
operator>>(unmarshall &u, wfs_slice &a)
{
  u >> a.val();
  return u;
}

inline marshall &
operator<<(marshall &m, wfs_slice a)
{
  m << a.val();
  return m;
}

#endif // __WFS_SLICE_H 1
