/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef __WFS_MAINT_SLICES_H__ 
#define __WFS_MAINT_SLICES_H__ 1

#include <set>
#include <slice_manager.h>
#include "wfs_maint_dirdisk.h"
#include "wfs_maint_protocol.h"

class wfs_maint_slices : public wfs_maint_dirdisk {

 private:

  slice_manager *_slices;
  std::map<wfs_slice_id, wfs_node> _tmp_replicas;
  std::set<wfs_slice_id> _possible_locked_slices;

  void get_replicas( wfs_file_handle fh, std::vector<wfs_node> &reps );
  void slices_maint_loop();
  wfs_node node_from_id( wfs_node_id id );
  int exchange_metadata_helper( wfs_node_id node_id, int new_rep,
				std::vector<wfs_maint_protocol::metadata> in_md,
				std::vector<wfs_maint_protocol::metadata> &out_md );
  bool do_metadata_exchange_slice( wfs_node node, wfs_slice_id slice );

 public:

  wfs_maint_slices( wfs_node me, rpcs *server, std::string dir, 
		    wfs_perfile_locker *locker, wfs_storage *storage,
		    wfs_client_manager *clients,
		    int maint_timer = 10 );
  ~wfs_maint_slices();

  void start_maint( slice_manager *slices, bool do_maint = true );

  int acquire_lock( wfs_slice_id sid, std::vector<host> hreps, int &dummy );
  int exchange_metadata_slice( wfs_node_id node_id, wfs_slice_id slice,
			       std::vector<wfs_maint_protocol::metadata> in_md,
			       std::vector<wfs_maint_protocol::metadata> &out_md );

};

#endif // __WFS_MAINT_SLICES_H__ 
