/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */

#include "canceller.h"
#include <assert.h>

canceller::canceller() {
  assert( pthread_mutex_init( &_lock, NULL ) == 0 );
}

canceller::~canceller() {
  assert( pthread_mutex_destroy( &_lock ) == 0 );
}

void canceller::cleanup_push( void (*handler)(void *), void *arg ) {

  assert( pthread_mutex_lock( &_lock ) == 0 );
  frame f;
  f.handler = handler;
  f.arg = arg;
  _stack.push_back(f);
  assert( pthread_mutex_unlock( &_lock ) == 0 );

}

void canceller::cleanup_pop( void (*handler)(void *), void *arg ) {
  assert( pthread_mutex_lock( &_lock ) == 0 );
  if( _stack.size() > 0 ) {
    std::vector<frame>::iterator i = _stack.end();
    do {
      i--;
      if( i->handler == handler && i->arg == arg ) {
	_stack.erase(i);
	break;
      }
    } while( i != _stack.begin() );
  }
  assert( pthread_mutex_unlock( &_lock ) == 0 );
}

void canceller::execute() {
  assert( pthread_mutex_lock( &_lock ) == 0 );
  if( _stack.size() > 0 ) {
    std::vector<frame>::iterator i = _stack.end();
    do {
      i--;
      (i->handler)(i->arg);
    } while( i != _stack.begin() );
    _stack.clear();
  }
  assert( pthread_mutex_unlock( &_lock ) == 0 );
}
