/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef ssh_chan_h
#define ssh_chan_h

// secure channel using libssh that handles identity verification with
// RSA keys and also encrypts connections.  Mostly drop-in replacement
// for tcpchan.

#ifdef WFS_WITH_LIBSSH

extern "C" {
#include <libssh/libssh.h>
#include <libssh/server.h>
}

#include <arpa/inet.h>

#include "base_chan.h"
#include "fifo.h"

#include "ident_util.h"

// internal. one SSH connection (client or server end).
// multiplexes send()s, un-multiplexes on recv().
class sshchan : public abstract_channel {
 public:

  sshchan(pthread_mutex_t *sshdelete_m, bool freebuf, canceller *cancel, 
	  SSH_SESSION *_session = NULL, int channo = -1);
  ~sshchan();

  int connect(const sockaddr_in &dst, ssh_cleanup_frame **frames);
  int start();

  int setup(sockaddr_in dst);
  int recv(charbuf &cb);
  bool dead();
  void lose();

  void close();

  const char *getpeername();
  int channo();

  bool verify_user(const char *username);
 private:
  CHANNEL *chan;
  SSH_SESSION *session;
  int fd;
  char authorized_user[MAX_IDENT_LENGTH+1];

  ssize_t xwritev(const struct iovec *iovptr, int iovcnt);
  int xread(void *p, unsigned int n, ssh_cleanup_frame **frames);

  void do_connect(sockaddr_in dst, short my_port);
};

class sshbind : public abstract_bind {
private:
  SSH_BIND *bind;
  int actual_port;
public:
  sshbind(SSH_BIND *_bind, int port) : bind(_bind), actual_port(port) {}
  sshchan *accept(canceller *cancel,
		  pthread_mutex_t *sshdelete_m,
		  bool freebuf);
  void close() {ssh_bind_free(bind);}
  unsigned int bound_port() {return actual_port;};
};

class sshhub : public chan_hub {
 public:
  sshhub();
  sshchan *channel(canceller *cancel,
		   pthread_mutex_t *sshdelete_m,
		   bool freebuf);

  sshbind *bind(const sockaddr_in &sin);
  int initial_timeout();
};

#endif
#endif
