#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/time.h>
#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/wait.h>
#include <dirent.h>
#include <string.h>
#include <fcntl.h>
#include <errno.h>
#include <assert.h>

/* fs-test-6 /mount1/file /mount2/file /mount3/file /mount4/file ...
 * test the cooperative client-side caching component of wfs
 * one client creates a big file
 * all other clients read the same file simultaneously*/
extern int errno;

typedef struct {
  char *f;
  char fill;
  int len;
  int usleep_time;
} check_arg;

int
create1(char *f, char fill, int len)
{
  int fd;
  char *p;

  p = (char *) malloc(len);
  assert(p);
  memset(p, fill, len);

  fd = creat(f, 0666);
  if(fd < 0){
    fprintf(stderr, "fs-test-6: create(%s): %s\n",
            f, strerror(errno));
    exit(1);
  }
  if(write(fd, p, len) != len){
    fprintf(stderr, "fs-test-6: write(%s): %s\n",
            f, strerror(errno));
    exit(1);
  }
  free(p);

  return fd;
}

void
check1(char *f, char fill, int len)
{
  int fd, cc, i;
  char *p;
  struct stat st;

  if(stat(f, &st) < 0){
    fprintf(stderr, "fs-test-6: stat(%s): %s\n",
            f, strerror(errno));
    exit(1);
  }
  if(st.st_size != len){
    fprintf(stderr, "fs-test-6: stat(%s) said len %d, wanted %d\n", 
            f, (int)st.st_size, len);
    exit(1);
  }

  fd = open(f, 0);
  if(fd < 0){
    fprintf(stderr, "fs-test-6: open(%s): %s\n",
            f, strerror(errno));
    exit(1);
  }

  p = malloc(len + 1024*32);
  assert(p);
  errno = 0;
  cc = read(fd, p, len + 1);
  if(cc != len){
    fprintf(stderr, "fs-test-6: read(%s) returned %d expected %d%s%s\n",
            f,
            cc,
            len,
            errno ? ": " : "",
            errno ? strerror(errno) : "");
    exit(1);
  }
  close(fd);

  for(i = 0; i < len; i++){
    if(p[i] != fill){
      fprintf(stderr, "fs-test-6: read 0x%02x expected 0x%02x\n",
              p[i] & 0xff, fill & 0xff);
      exit(1);
    }
  }
  free(p);
}

void *
check(void *pp)
{
  check_arg *p = (check_arg *)pp;
  usleep(p->usleep_time);
  check1(p->f, p->fill, p->len);
  return NULL;
}

#define FILESZ 10000
int
main(int argc, char *argv[])
{
  int fd, i;
  check_arg *arglist;
  pthread_t *t;
  struct timeval tv1, tv2;
  long long diff;

  if (argc < 3) {
	 fprintf(stderr,"usage: fs-test-6 file1 file file file ...\n");
	 exit(1);
  }
  for (i = 0; i < argc; i++)
	 fprintf(stderr, "%s ", argv[i]);
  fprintf(stderr, "\n");

  fd = create1(argv[1], 'a', FILESZ);
  close(fd);
  check1(argv[1], 'a', FILESZ);

  t = (pthread_t *) malloc(sizeof(pthread_t)*(argc-2));
  arglist = (check_arg *)malloc(sizeof(check_arg)*(argc-2));

  gettimeofday(&tv1, NULL);
  for (i = 2; i < argc; i++)  {
	 arglist[i-2].f = argv[i];
	 arglist[i-2].fill = 'a';
	 arglist[i-2].len = FILESZ;
	 arglist[i-2].usleep_time = i*100000;
	 pthread_create(&(t[i-2]), NULL, check, (void *)&(arglist[i-2]));
  }

  for (i = 2; i < argc; i++) 
	 pthread_join(t[i-2]);
  gettimeofday(&tv2, NULL);

  if (tv2.tv_sec > tv1.tv_sec)
	 diff = (tv2.tv_sec-tv1.tv_sec)*1000000 - (tv2.tv_usec-tv1.tv_usec);
  else
	 diff = (tv2.tv_usec-tv1.tv_usec);
  fprintf(stderr,"fs-test-6: passed, finished all reads in %ld us\n", diff);
  return 0;

}
