/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#include "wfs_file.h"
#include "wfs_misc.h"
#include <assert.h>

int
main(int argc, char *argv[])
{

  wfs_set_debug(argc-1);

  wfs_file_handle fh;
  uint16_t orig_cues = fh.which_cues;

  printf ("*** Testing search_and_set_cues ***\n");

  printf("Testing dotless path component... ");
  assert (search_and_set_cues("pineapple",fh) == false);
  assert (fh.which_cues == orig_cues);
  printf("OK\n");

  printf("Testing dotted noncue path component... ");
  assert (search_and_set_cues(".wombat",fh) == false);
  assert (fh.which_cues == orig_cues);
  printf("OK\n");

  printf("Testing valid basic cue... ");
  assert (search_and_set_cues(".versioned",fh) == true);
  assert (fh.get_cue(WFS_CUE_VERSIONING).val == WFS_CUE_VERSIONED);
  printf("OK\n");

  printf("Testing valid parameterized cue... ");
  assert (search_and_set_cues(".version=4",fh) == true);
  int res = fh.get_cue(WFS_CUE_VERSION_PARAM).val;
  if (res != 4) {
    printf("%d != 4!\n",res);
    exit(1);
  }
  printf("OK\n");

  printf ("*** Testing accessors ***\n");
  wfs_file_handle fh2;

  printf("Testing fhlease... ");
  assert(fhlease(fh2) == (signed)WFS_LEASE_SEC_DEFAULT);
  assert(search_and_set_cues(".2000.lease",fh2) == true);
  assert(fhlease(fh2) == 2);
  assert(search_and_set_cues(".lease=1000",fh2) == true);
  assert(fhlease(fh2) == 1);
  printf("OK\n");

  printf("Testing slot_from_ind...");
  assert(wfs_file_handle::slot_from_ind(WFS_CUE_CONSISTENCY) 
	 != wfs_file_handle::slot_from_ind(WFS_CUE_MAX_TIME));
  assert(wfs_file_handle::ind_from_slot(wfs_file_handle::slot_from_ind(WFS_CUE_CONSISTENCY)) == WFS_CUE_CONSISTENCY);
  printf("OK\n");

  printf("wfs_file-unit-tests: Passed all tests.\n");

  exit(0);
  return(0);
}
