/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#include "chash.h"

#include <assert.h>
#include <stdlib.h>

wfs_node_id sum_of_node_ids( wfs_node_id a, wfs_node_id b ) {
  return a+b; // let overflow take care of it?
}

wfs_node_id diff_between_node_ids( wfs_node_id start_id, 
				   wfs_node_id end_id ) {

  if( end_id < start_id ) {
    wfs_node_id max = 0xffffffff;
    max <<= 32;
    max |= 0xffffffff;
    return end_id + (max - start_id);
  } else {
    return end_id - start_id;
  }

}

wfs_node_id rand_node_id() {
  // the last four bits of random() are known to be not-so-random, so
  // shift them off.
  wfs_node_id rand_id = (random() >> 4);
  rand_id <<= 36;
  rand_id |= ((random() >> 4) << 4);
  rand_id |= (random() >> 28);
  return rand_id;
}

wfs_file_id rand_file_id_between( wfs_node_id start_id, wfs_node_id end_id ) {
  return (node_id_to_file_id( rand_node_id_between( start_id, end_id ) ));
}

wfs_node_id rand_node_id_between( wfs_node_id start_id, wfs_node_id end_id ) {

  assert( start_id != end_id );

  // TODO: for now, just pick a random id.  later we'll probably want
  // to skew it to be closer to the end_id.
  wfs_node_id diff = diff_between_node_ids( start_id, end_id );
  wfs_node_id offset = rand_node_id() % diff;
  return sum_of_node_ids( start_id, offset );

}

bool between_ids_incl( wfs_node_id start, wfs_node_id end, wfs_node_id id ) {

  if( (start == end) && (id == start)) {
    return true;
  } else if( start < end ) {
    return (id >= start) && (id <= end);
  } else {
    return (id >= start) || (id <= end);
  }

}
