/**
 * Copyright (c) 2009 Jeremy Stribling, Yair Sovran, Irene Zhang, 
 *                    Xavid Pretzer, Jinyang Li, Frans Kaashoek, Robert Morris,
 *     		      Massachusetts Institute of Technology,
 * 		      New York University
 * 
 * Permission is hereby granted, free of charge, to any person obtaining
 * a copy of this software and associated documentation files (the
 * "Software"), to deal in the Software without restriction, including
 * without limitation the rights to use, copy, modify, merge, publish,
 * distribute, sublicense, and/or sell copies of the Software, and to
 * permit persons to whom the Software is furnished to do so, subject to
 * the following conditions:
 * 
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 * 
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
 * LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
 * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 */


#ifndef __WFS_STORAGE_MMAP_H__
#define __WFS_STORAGE_MMAP_H__ 1

#include "wfs_storage.h"

class wfs_storage_mmap : public wfs_storage {

 protected:

  typedef struct mmap_dir {
    char *mapped;
    int fd;
    int off;
    int size;
    void *not_mmap;
  } mmap_dir;

  bool find_fid( void *dirp, const std::string &name, 
		 char *line, bool use_line, 
		 wfs_file_id &fid, bool &higher,
		 char **line_buf, int &line_size );
  void * open_finder( wfs_file_handle fh );
  void close_finder( void *dirp );
  int read_from_finder( void *filep, char *buf, int size, 
			const unsigned char **blockp );

 public:

  wfs_storage_mmap( std::string dir );
  virtual ~wfs_storage_mmap();

  bool readfile( wfs_file_handle fh, wfs_version version, 
  		 unsigned long long off, int n, std::string &buf );
  bool writefile( wfs_file_handle fh, 
		  std::vector<unsigned long long> off, 
		  std::vector<std::string> buf );

};

#endif // __WFS_STORAGE_MMAP_H
